<?php
//==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>
//
// Ajax簡易投票所 v3.25 [ GPL ]
// Copyright (c) phpkobo.com ( http://jpn.phpkobo.com/ )
// Email : admin@phpkobo.com
// ID : APSMX-325J
//
//==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<

class CDataConn extends CDataConnBase {

	public function __construct( $sys, $prt ) {
		parent::__construct( $sys, $prt );
	}

	public function setup() {
		$name = $this->getCookieBlockName();
		$this->ckblock = new CCookieBlock();
		$this->ckblock->setup( $name );

		return true;
	}

	public function getDataFolderPath() {
		return $this->sys->getAppDataPath() . "std." . $this->sys->getIdName() . '/';
	}

	private function getDataFilePath() {
		return $this->getDataFolderPath() . "votes.txt";
	}

	private function getDataTxt() {
		$ax = array();
		foreach( $this->prt->items as $key => $item ) {
			$ax[] = "{$key}=" . $item->getCount();
		}
		return implode( "\r\n", $ax );
	}

	private function getIPBlockFilePath() {
		return $this->getDataFolderPath() . "ip-block.txt";
	}

	private function getCookieBlockName() {
		return $this->sys->getIdName() . ".cookie-block";
	}

	private function checkDataFilePerm( $path ) {
		if ( !file_exists( $path ) ) {
			if ( is_writable( dirname($path) ) ) {
				touch( $path );
			} else {
				return $this->sys->checkPermission( 'rw', dirname($path) );
			}
		}
		return $this->sys->checkPermission( 'rw', $path );
	}

	public function hasVoted( $name = null ) {

		if ( $this->prt->attr( "enable-cookie-block" ) ) {
			if ( is_null( $name ) ) {
				return $this->ckblock->exists();
			} else {
				if ( $this->ckblock->exists() ) {
					return in_array( $name,
						explode( "\t", $this->ckblock->get() ) );
				} else {
					return false;
				}
			}
		} else {
			return false;
		}
	}

	public function load2( $b_vote ) {

		$path = $this->getDataFilePath();
		if ( !file_exists( $path ) ) { @touch( $path ); }
		if ( !$this->sys->checkPermission( 'rw', $path ) ) { return false; }

		$handle = @fopen( $path, "r+" );

		//-- do an exclusive lock
		if ( !( @flock( $handle, LOCK_EX ) ) ) {
			$this->sys->showErrorMsg( "Can not write [app.data] ({$path})" .
				" (Could not get the lock!)" );
			return false;
		}

		$txt = '';
		$size = filesize( $path );
		if ( $size > 0 ) {
			$txt = fread( $handle, $size );
		}

		$items =& $this->prt->items;

		//-- load into CPollItem
		$txt = str_replace( "\r", "", $txt );
		$ax = explode( "\n", $txt );
		foreach( $ax as $ln ) {
			if ( strpos( $ln, "=" ) !== false ) {
				$bx = explode( "=", $ln );
				if ( array_key_exists( $bx[0], $items ) ) {
					$item = $items[ $bx[0] ];
					$item->setCount( $bx[1] );
				}
			}
		}

		//-- add votes
		$answer = isset( $_REQUEST['answer'] ) ? $_REQUEST['answer'] : "";
		$answer = json_decode( $answer, true );
		if ( $b_vote && !empty($answer) ) {
			$b_found = false;
			foreach( $answer as $ans ) {
				if ( isset( $items[$ans] ) ) {
					$item = $items[$ans];
					$item->incCount();
					$b_found = true;
				}
			}

			if ( $b_found ) {
				//-- truncate file
				fseek( $handle, 0 );
				ftruncate( $handle, 0 );

				//-- write votes to txt file
				fwrite( $handle, $this->getDataTxt() );
			} else {
				$this->prt->showErrorMsg( "Item ({$answer}) not declared" );
			}
		}

		//-- release the lock
		flock( $handle, LOCK_UN );

		//-- close the file
		fclose($handle);

		return true;
	}

	public function load( &$ret, $b_vote ) {

		if ( $b_vote ) {

			//-- IP Block
			if ( $this->prt->attr( "enable-ip-block" ) ) {
				$path = $this->getIPBlockFilePath();
				if ( !$this->checkDataFilePerm( $path ) ) { return false; }

				$ipblock = new CIPBlock();
				$ipblock->setup( $path );
				if ( $ipblock->exists() ) {
					$ret["cmd"] = "none";
					$ret["msg"] = array( "cmd" => "already_voted" );
					return false;
				}
			}

			//-- Cookie Block
			if ( $this->prt->attr( "enable-cookie-block" ) ) {
				$cookie_name = $this->getCookieBlockName();
				$cookie_block = new CCookieBlock();
				$cookie_block->setup( $cookie_name );
				if ( $cookie_block->exists() ) {
					$ret["cmd"] = "none";
					$ret["msg"] = array( "cmd" => "already_voted" );
					return false;
				}
			}
		}

		//-- Load Data
		$this->load2(true);

		if ( $b_vote ) {

			//-- record ip
			if ( isset( $ipblock ) ) {
				$ipblock->add();
			}

			//-- store choice in cookie
			if ( isset( $cookie_block ) ) {
				$answer = isset( $_REQUEST['answer'] ) ? $_REQUEST['answer'] : "";
				$answer = json_decode( $answer, true );
				$val = implode( "\t", $answer );
				$cookie_block->add( $val );
			}

			//-- return response
			$ret["msg"] = array( "cmd" => "thank_you" );
		}

		return true;
	}

	public function clear_block( &$ret ) {

		$ret["cmd"] = "alert";

		if ( $this->prt->attr( "b-reset-block" ) ) {

			CTClassSys::loadLang('ajax.block',$lng);
			$msg_ip_block_reset = $lng[ 'msg:ip-block-reset' ];
			$msg_ip_block_not_reset = $lng[ 'msg:ip-block-not-reset' ];
			$msg_cookie_block_reset = $lng[ 'msg:cookie-block-reset' ];
			$msg_cookie_block_not_reset = $lng[ 'msg:cookie-block-not-reset' ];

			//-- IP Block
			$path = $this->getIPBlockFilePath();
			if ( !$this->checkDataFilePerm( $path ) ) return false;
			$ipblock = new CIPBlock();
			$ipblock->setup( $path );
			if ( $ipblock->clear() ) {
				echo $msg_ip_block_reset . "\r\n";
			} else {
				echo $msg_ip_block_not_reset . "\r\n";
			}

			//-- Cookie Block
			$cookie_name = $this->getCookieBlockName();
			$cookie_block = new CCookieBlock();
			$cookie_block->setup( $cookie_name );
			if ( $cookie_block->clear() ) {
				echo $msg_cookie_block_reset . "\r\n";
			} else {
				echo $msg_cookie_block_not_reset . "\r\n";
			}
		} else {
			echo "Unauthorized Access\r\n";
		}

		return true;
	}

}

?>