<?php
//==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>
//
// Ajax簡易投票所 v3.25 [ GPL ]
// Copyright (c) phpkobo.com ( http://jpn.phpkobo.com/ )
// Email : admin@phpkobo.com
// ID : APSMX-325J
//
//==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<

class CPoll {

	public function __construct( $sys, $prt ) {
		//-- pointers
		$this->sys =& $sys;
		$this->prt =& $prt;

		//-- attributes
		$this->_attr_ = array();

		//-- poll items
		$this->items = array();
		$this->item_key_list = array();
		$this->item_key_idx = 0;

		//-- default max width
		$this->setMaxWidth( 100 );
	}

	public function setup() {

		//-- data connection
		switch ( $this->attr('data-conn') ) {
		case "atr":
			$path = dirname(dirname(__FILE__)) . "/dc-atr/index.inc.php";
			break;

		case "std":
		default:
			$path = dirname(dirname(__FILE__)) . "/dc-std/index.inc.php";
			break;
		}
		if ( !file_exists($path) ) {
			$this->sys->fatalError("Addon does not exist (".dirname($path).")");
		}
		include( $path );
		$this->dataconn = new CDataConn($this->sys,$this);
		if ( !$this->sys->setupDataFolder(
			$this->dataconn->getDataFolderPath()) ) { return false; }
		if ( !$this->dataconn->setup() ) { return false; }

		return true;
	}

	public function attr( $key, $val = null ) {
		if ( is_null( $val ) ) {
			if ( isset( $this->_attr_[$key] ) ) {
				return $this->_attr_[$key];
			} else {
				return null;
			}
		} else {
			$this->_attr_[$key] = $val;
		}
	}

	public static function renderCountTpl( $n, $tpl ) {
		$s = str_replace( "%n%", $n, $tpl );
		$s = str_replace( "%s%", ( $n == 1 ) ? "" : "s", $s );
		return $s;
	}

	public function hasVoted( $key = null ) {
		return $this->dataconn->hasVoted( $key );
	}

	public function getTotal() {
		return $this->total;
	}

	public function setMaxWidth( $max_width ) {
		$this->max_width = $max_width;
	}

	public function getMaxWidth() {
		return $this->max_width;
	}

	public function getMaxCount() {
		return $this->max_count;
	}

	public function getNumOfItems() {
		return count( $this->items );
	}

	public function sortItems($a, $b) {
		$acnt = $a->getCount();
		$bcnt = $b->getCount();
		if ( $acnt == $bcnt ) {
			return 0;
		}
		if ( $this->orderby == 'asc' ) {
			return ($acnt < $bcnt) ? -1 : 1;
		} else {
			return ($acnt < $bcnt) ? 1 : -1;
		}
	}

	public function getAllItems( $orderby = "def" ) {
		switch ( $orderby ) {
		case "def":
		default:
			return $this->items;
		case "asc":
		case "desc":
			$this->orderby = $orderby;
			$items = $this->items;
			uasort($items, array($this,'sortItems'));
			return $items;
		}
	}

	public function getItem( $key ) {
		$err_msg = array();
		if ( !isset( $this->items[$key] ) ) {
			$err_msg[] = "getItem( key ) : Unknown Item Key ({$key})";
		}

		if ( count( $err_msg ) > 0 ) {
			$this->sys->showErrorMsg( $err_msg );
		}

		return $this->items[$key];
	}

	public function addItem( $key, $val ) {
		$item = new CPollItem();
		$item->setup( $this->sys, $this, $key, $val );
		$this->items[$key] = $item;
		$this->item_key_list[$this->item_key_idx] = $key;
		$this->item_key_idx++;
		return $item;
	}

	public function calcStats() {
		$total = 0;
		$max_count = 0;
		foreach( $this->items as $key => $item ) {
			$count = $item->getCount();
			$total += $count;
			if ( $max_count < $count ) $max_count = $count;
		}
		$this->total = $total;
		$this->max_count = $max_count;
	}

	public function getFolderUrl() {
		return $this->sys->getFolderUrl();
	}

	public function started() {
		return !(
			( !empty( $this->_attr_[ 'dt-start' ] ) ) &&
			( strtotime( $this->_attr_[ 'dt-start' ] ) > time() )
		);
	}

	public function ended() {
		return (
			( !empty( $this->_attr_[ 'dt-end' ] ) ) &&
			( strtotime( $this->_attr_[ 'dt-end' ] ) < time() )
		);
	}

	public function load( &$ret, $b_vote ) {
		if ( $this->dataconn->load( $ret, $b_vote ) ) {
			$this->calcStats();
			return true;
		}
		return false;
	}

	public function clear_block( &$ret ) {
		if ( !$this->attr( "b-reset-block" ) ) {
			echo "Unauthorized Access\r\n";
			return true;
		}

		return $this->dataconn->clear_block( $ret );
	}

}

?>